/**
 * DevCycle Bucketing API
 * Documents the DevCycle Bucketing API which provides and API interface to User Bucketing and for generated SDKs.
 *
 * OpenAPI spec version: 1.0.0
 *
 *
 * NOTE: This file is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the file manually.
 */
/// <reference path="custom.d.ts" />
import { Configuration } from "./configuration";
/**
 *
 * @export
 */
export declare const COLLECTION_FORMATS: {
    csv: string;
    ssv: string;
    tsv: string;
    pipes: string;
};
/**
 *
 * @export
 * @interface FetchAPI
 */
export interface FetchAPI {
    (url: string, init?: any): Promise<Response>;
}
/**
 *
 * @export
 * @interface FetchArgs
 */
export interface FetchArgs {
    url: string;
    options: any;
}
/**
 *
 * @export
 * @class BaseAPI
 */
export declare class BaseAPI {
    protected basePath: string;
    protected fetch: FetchAPI;
    protected configuration: Configuration;
    constructor(configuration: Configuration, basePath?: string, fetch?: FetchAPI);
}
/**
 *
 * @export
 * @class RequiredError
 * @extends {Error}
 */
export declare class RequiredError extends Error {
    field: string;
    constructor(field: string, msg?: string);
}
/**
 *
 * @export
 * @interface ErrorResponse
 */
export interface ErrorResponse {
    /**
     * Error message
     * @type {string}
     * @memberof ErrorResponse
     */
    message: string;
    /**
     * Additional error information detailing the error reasoning
     * @type {any}
     * @memberof ErrorResponse
     */
    data?: any;
}
/**
 *
 * @export
 * @interface Event
 */
export interface Event {
    /**
     * Custom event type
     * @type {string}
     * @memberof Event
     */
    type: string;
    /**
     * Custom event target / subject of event. Contextual to event type
     * @type {string}
     * @memberof Event
     */
    target?: string;
    /**
     * Unix epoch time the event occurred according to client
     * @type {number}
     * @memberof Event
     */
    date?: number;
    /**
     * Value for numerical events. Contextual to event type
     * @type {number}
     * @memberof Event
     */
    value?: number;
    /**
     * Extra JSON metadata for event. Contextual to event type
     * @type {any}
     * @memberof Event
     */
    metaData?: any;
}
/**
 *
 * @export
 * @interface Feature
 */
export interface Feature {
    /**
     * unique database id
     * @type {string}
     * @memberof Feature
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Feature
     */
    key: string;
    /**
     * Feature type
     * @type {string}
     * @memberof Feature
     */
    type: Feature.TypeEnum;
    /**
     * Bucketed feature variation
     * @type {string}
     * @memberof Feature
     */
    _variation: string;
    /**
     * Evaluation reasoning
     * @type {string}
     * @memberof Feature
     */
    evalReason?: string;
}
/**
 * @export
 * @namespace Feature
 */
export declare namespace Feature {
    /**
     * @export
     * @enum {string}
     */
    enum TypeEnum {
        Release,
        Experiment,
        Permission,
        Ops
    }
}
/**
 *
 * @export
 * @interface InlineResponse201
 */
export interface InlineResponse201 {
    /**
     *
     * @type {string}
     * @memberof InlineResponse201
     */
    message?: string;
}
/**
 *
 * @export
 * @interface UserData
 */
export interface UserData {
    /**
     * Unique id to identify the user
     * @type {string}
     * @memberof UserData
     */
    user_id: string;
    /**
     * User's email used to identify the user on the dashboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    email?: string;
    /**
     * User's name used to identify the user on the dashboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    name?: string;
    /**
     * User's language in ISO 639-1 format
     * @type {string}
     * @memberof UserData
     */
    language?: string;
    /**
     * User's country in ISO 3166 alpha-2 format
     * @type {string}
     * @memberof UserData
     */
    country?: string;
    /**
     * App Version of the running application
     * @type {string}
     * @memberof UserData
     */
    appVersion?: string;
    /**
     * App Build number of the running application
     * @type {string}
     * @memberof UserData
     */
    appBuild?: string;
    /**
     * User's custom data to target the user with, data will be logged to DevCycle for use in dashboard.
     * @type {any}
     * @memberof UserData
     */
    customData?: any;
    /**
     * User's custom data to target the user with, data will not be logged to DevCycle only used for feature bucketing.
     * @type {any}
     * @memberof UserData
     */
    privateCustomData?: any;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    createdDate?: number;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    lastSeenDate?: number;
    /**
     * Platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platform?: string;
    /**
     * Version of the platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platformVersion?: string;
    /**
     * User's device type
     * @type {string}
     * @memberof UserData
     */
    deviceType?: string;
    /**
     * DevCycle SDK type
     * @type {string}
     * @memberof UserData
     */
    sdkType?: UserData.SdkTypeEnum;
    /**
     * DevCycle SDK Version
     * @type {string}
     * @memberof UserData
     */
    sdkVersion?: string;
}
/**
 * @export
 * @namespace UserData
 */
export declare namespace UserData {
    /**
     * @export
     * @enum {string}
     */
    enum SdkTypeEnum {
        Api,
        Server
    }
}
/**
 *
 * @export
 * @interface UserDataAndEventsBody
 */
export interface UserDataAndEventsBody {
    /**
     *
     * @type {Array<Event>}
     * @memberof UserDataAndEventsBody
     */
    events?: Array<Event>;
    /**
     *
     * @type {UserData}
     * @memberof UserDataAndEventsBody
     */
    user?: UserData;
}
/**
 *
 * @export
 * @interface Variable
 */
export interface Variable {
    /**
     * unique database id
     * @type {string}
     * @memberof Variable
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Variable
     */
    key: string;
    /**
     * Variable type
     * @type {string}
     * @memberof Variable
     */
    type: Variable.TypeEnum;
    /**
     * Variable value can be a string, number, boolean, or JSON
     * @type {any}
     * @memberof Variable
     */
    value: any;
}
/**
 * @export
 * @namespace Variable
 */
export declare namespace Variable {
    /**
     * @export
     * @enum {string}
     */
    enum TypeEnum {
        String,
        Boolean,
        Number,
        JSON
    }
}
/**
 * DVC - object-oriented interface
 * @export
 * @class DVC
 * @extends {BaseAPI}
 */
export declare class DVC extends BaseAPI {
    /**
     *
     * @summary Get all features by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    getFeatures(body: UserData, options?: any): Promise<{
        [key: string]: Feature;
    }>;
    /**
     *
     * @summary Get variable by key for user data
     * @param {UserData} body
     * @param {string} key Variable key
     * @param {any} defaultValue Value returned if request fails or doesn't return a value for the key
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    getVariableByKey(body: UserData, key: string, defaultValue: any, options?: any): Promise<Variable>;
    /**
     *
     * @summary Get all variables by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    getVariables(body: UserData, options?: any): Promise<{
        [key: string]: Variable;
    }>;
    /**
     *
     * @summary Post events to DevCycle for user
     * @param {UserDataAndEventsBody} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    postEvents(body: UserDataAndEventsBody, options?: any): Promise<InlineResponse201>;
}
