/// <reference path="./custom.d.ts" />
// tslint:disable
/**
 * DevCycle Bucketing API
 * Documents the DevCycle Bucketing API which provides and API interface to User Bucketing and for generated SDKs.
 *
 * OpenAPI spec version: 1.0.0
 *
 *
 * NOTE: This file is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the file manually.
 */

import * as url from "url";
import * as nodeFetch from "node-fetch";
import { Configuration } from "./configuration";

const BASE_PATH = "https://bucketing-api.devcycle.com/".replace(/\/+$/, "");

/**
 *
 * @export
 */
export const COLLECTION_FORMATS = {
    csv: ",",
    ssv: " ",
    tsv: "\t",
    pipes: "|",
};

/**
 *
 * @export
 * @interface FetchAPI
 */
export interface FetchAPI {
    (url: string, init?: any): Promise<Response>;
}

/**
 *
 * @export
 * @interface FetchArgs
 */
export interface FetchArgs {
    url: string;
    options: any;
}

/**
 *
 * @export
 * @class BaseAPI
 */
export class BaseAPI {
    protected configuration: Configuration

    constructor(configuration: Configuration, protected basePath: string = BASE_PATH, protected fetch: FetchAPI = nodeFetch) {
        this.configuration = configuration;
        this.basePath = configuration.basePath || this.basePath;
    }
}

/**
 *
 * @export
 * @class RequiredError
 * @extends {Error}
 */
export class RequiredError extends Error {
    constructor(public field: string, msg?: string) {
        super(msg);
    }
}

/**
 *
 * @export
 * @interface ErrorResponse
 */
export interface ErrorResponse {
    /**
     * Error message
     * @type {string}
     * @memberof ErrorResponse
     */
    message: string;
    /**
     * Additional error information detailing the error reasoning
     * @type {any}
     * @memberof ErrorResponse
     */
    data?: any;
}
/**
 *
 * @export
 * @interface Event
 */
export interface Event {
    /**
     * Custom event type
     * @type {string}
     * @memberof Event
     */
    type: string;
    /**
     * Custom event target / subject of event. Contextual to event type
     * @type {string}
     * @memberof Event
     */
    target?: string;
    /**
     * Unix epoch time the event occurred according to client
     * @type {number}
     * @memberof Event
     */
    date?: number;
    /**
     * Value for numerical events. Contextual to event type
     * @type {number}
     * @memberof Event
     */
    value?: number;
    /**
     * Extra JSON metadata for event. Contextual to event type
     * @type {any}
     * @memberof Event
     */
    metaData?: any;
}
/**
 *
 * @export
 * @interface Feature
 */
export interface Feature {
    /**
     * unique database id
     * @type {string}
     * @memberof Feature
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Feature
     */
    key: string;
    /**
     * Feature type
     * @type {string}
     * @memberof Feature
     */
    type: Feature.TypeEnum;
    /**
     * Bucketed feature variation
     * @type {string}
     * @memberof Feature
     */
    _variation: string;
    /**
     * Evaluation reasoning
     * @type {string}
     * @memberof Feature
     */
    evalReason?: string;
}

/**
 * @export
 * @namespace Feature
 */
export namespace Feature {
    /**
     * @export
     * @enum {string}
     */
    export enum TypeEnum {
        Release = <any> 'release',
        Experiment = <any> 'experiment',
        Permission = <any> 'permission',
        Ops = <any> 'ops'
    }
}
/**
 *
 * @export
 * @interface InlineResponse201
 */
export interface InlineResponse201 {
    /**
     *
     * @type {string}
     * @memberof InlineResponse201
     */
    message?: string;
}
/**
 *
 * @export
 * @interface UserData
 */
export interface UserData {
    /**
     * Unique id to identify the user
     * @type {string}
     * @memberof UserData
     */
    user_id: string;
    /**
     * User's email used to identify the user on the dashboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    email?: string;
    /**
     * User's name used to identify the user on the dashboard / target audiences
     * @type {string}
     * @memberof UserData
     */
    name?: string;
    /**
     * User's language in ISO 639-1 format
     * @type {string}
     * @memberof UserData
     */
    language?: string;
    /**
     * User's country in ISO 3166 alpha-2 format
     * @type {string}
     * @memberof UserData
     */
    country?: string;
    /**
     * App Version of the running application
     * @type {string}
     * @memberof UserData
     */
    appVersion?: string;
    /**
     * App Build number of the running application
     * @type {string}
     * @memberof UserData
     */
    appBuild?: string;
    /**
     * User's custom data to target the user with, data will be logged to DevCycle for use in dashboard.
     * @type {any}
     * @memberof UserData
     */
    customData?: any;
    /**
     * User's custom data to target the user with, data will not be logged to DevCycle only used for feature bucketing.
     * @type {any}
     * @memberof UserData
     */
    privateCustomData?: any;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    createdDate?: number;
    /**
     * Date the user was created, Unix epoch timestamp format
     * @type {number}
     * @memberof UserData
     */
    lastSeenDate?: number;
    /**
     * Platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platform?: string;
    /**
     * Version of the platform the Client SDK is running on
     * @type {string}
     * @memberof UserData
     */
    platformVersion?: string;
    /**
     * User's device type
     * @type {string}
     * @memberof UserData
     */
    deviceType?: string;
    /**
     * DevCycle SDK type
     * @type {string}
     * @memberof UserData
     */
    sdkType?: UserData.SdkTypeEnum;
    /**
     * DevCycle SDK Version
     * @type {string}
     * @memberof UserData
     */
    sdkVersion?: string;
}

/**
 * @export
 * @namespace UserData
 */
export namespace UserData {
    /**
     * @export
     * @enum {string}
     */
    export enum SdkTypeEnum {
        Api = <any> 'api',
        Server = <any> 'server'
    }
}
/**
 *
 * @export
 * @interface UserDataAndEventsBody
 */
export interface UserDataAndEventsBody {
    /**
     *
     * @type {Array<Event>}
     * @memberof UserDataAndEventsBody
     */
    events?: Array<Event>;
    /**
     *
     * @type {UserData}
     * @memberof UserDataAndEventsBody
     */
    user?: UserData;
}
/**
 *
 * @export
 * @interface Variable
 */
export interface Variable {
    /**
     * unique database id
     * @type {string}
     * @memberof Variable
     */
    _id: string;
    /**
     * Unique key by Project, can be used in the SDK / API to reference by 'key' rather than _id.
     * @type {string}
     * @memberof Variable
     */
    key: string;
    /**
     * Variable type
     * @type {string}
     * @memberof Variable
     */
    type: Variable.TypeEnum;
    /**
     * Variable value can be a string, number, boolean, or JSON
     * @type {any}
     * @memberof Variable
     */
    value: any;
}

/**
 * @export
 * @namespace Variable
 */
export namespace Variable {
    /**
     * @export
     * @enum {string}
     */
    export enum TypeEnum {
        String = <any> 'String',
        Boolean = <any> 'Boolean',
        Number = <any> 'Number',
        JSON = <any> 'JSON'
    }
}

/**
 * DVC - fetch parameter creator
 * @export
 */
const DVCFetchParamCreator = function (configuration: Configuration) {
    return {
        /**
         *
         * @summary Get all features by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getFeatures(body: UserData, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getFeatures.');
            }
            const userData = {
                ...body,
                ...configuration.defaultUserData
            }
            const localVarPath = `/v1/features`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(userData) : (userData);

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Get variable by key for user data
         * @param {UserData} body
         * @param {string} key Variable key
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariableByKey(body: UserData, key: string, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getVariableByKey.');
            }
            // verify required parameter 'key' is not null or undefined
            if (key === null || key === undefined) {
                throw new RequiredError('key','Required parameter key was null or undefined when calling getVariableByKey.');
            }
            const userData = {
                ...body,
                ...configuration.defaultUserData
            }
            const localVarPath = `/v1/variables/{key}`
                .replace(`{${"key"}}`, encodeURIComponent(String(key)));
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(userData) : (userData);

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Get all variables by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariables(body: UserData, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling getVariables.');
            }
            const userData = {
                ...body,
                ...configuration.defaultUserData
            }
            const localVarPath = `/v1/variables`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserData" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(userData) : (userData);

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
        /**
         *
         * @summary Post events to DevCycle for user
         * @param {UserDataAndEventsBody} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        postEvents(body: UserDataAndEventsBody, options: any = {}): FetchArgs {
            // verify required parameter 'body' is not null or undefined
            if (body === null || body === undefined) {
                throw new RequiredError('body','Required parameter body was null or undefined when calling postEvents.');
            }

            const userAndEventsData = {
                events: body.events,
                user: {
                    ...body.user,
                    ...configuration.defaultUserData
                }
            }
            const localVarPath = `/v1/track`;
            const localVarUrlObj = url.parse(localVarPath, true);
            const localVarRequestOptions = Object.assign({ method: 'POST' }, options);
            const localVarHeaderParameter = {} as any;
            const localVarQueryParameter = {} as any;

            // authentication bearerAuth required
            if (configuration && configuration.apiKey) {
                const localVarApiKeyValue = typeof configuration.apiKey === 'function'
					? configuration.apiKey("Authorization")
					: configuration.apiKey;
                localVarHeaderParameter["Authorization"] = localVarApiKeyValue;
            }

            localVarHeaderParameter['Content-Type'] = 'application/json';

            localVarUrlObj.query = Object.assign({}, localVarUrlObj.query, localVarQueryParameter, options.query);
            // fix override query string Detail: https://stackoverflow.com/a/7517673/1077943
            delete localVarUrlObj.search;
            localVarRequestOptions.headers = Object.assign({}, localVarHeaderParameter, options.headers);
            const needsSerialization = (<any>"UserDataAndEventsBody" !== "string") || localVarRequestOptions.headers['Content-Type'] === 'application/json';
            localVarRequestOptions.body =  needsSerialization ? JSON.stringify(userAndEventsData) : (userAndEventsData);

            return {
                url: url.format(localVarUrlObj),
                options: localVarRequestOptions,
            };
        },
    }
};

/**
 * DVC - functional programming interface
 * @export
 */
const DVCFp = function(configuration: Configuration) {
    return {
        /**
         *
         * @summary Get all features by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getFeatures(body: UserData, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<{ [key: string]: Feature; }> {
            const localVarFetchArgs = DVCFetchParamCreator(configuration).getFeatures(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
        /**
         *
         * @summary Get variable by key for user data
         * @param {UserData} body
         * @param {string} key Variable key
         * @param {any} defaultValue Value returned if request fails or doesn't return a value for the key
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariableByKey(body: UserData, key: string, defaultValue: any, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<Variable> {
            const localVarFetchArgs = DVCFetchParamCreator(configuration).getVariableByKey(body, key, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        console.error(response)
                        return {
                            value: defaultValue,
                            key
                        }
                    }
                });
            };
        },
        /**
         *
         * @summary Get all variables by key for user data
         * @param {UserData} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        getVariables(body: UserData, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<{ [key: string]: Variable; }> {
            const localVarFetchArgs = DVCFetchParamCreator(configuration).getVariables(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
        /**
         *
         * @summary Post events to DevCycle for user
         * @param {UserDataAndEventsBody} body
         * @param {*} [options] Override http request option.
         * @throws {RequiredError}
         */
        postEvents(body: UserDataAndEventsBody, options?: any): (fetch?: FetchAPI, basePath?: string) => Promise<InlineResponse201> {
            const localVarFetchArgs = DVCFetchParamCreator(configuration).postEvents(body, options);
            return (fetch: FetchAPI = nodeFetch, basePath: string = BASE_PATH) => {
                return fetch(basePath + localVarFetchArgs.url, localVarFetchArgs.options).then((response) => {
                    if (response.status >= 200 && response.status < 300) {
                        return response.json();
                    } else {
                        throw response;
                    }
                });
            };
        },
    }
};

/**
 * DVC - object-oriented interface
 * @export
 * @class DVC
 * @extends {BaseAPI}
 */
export class DVC extends BaseAPI {
    /**
     *
     * @summary Get all features by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getFeatures(body: UserData, options?: any) {
        return DVCFp(this.configuration).getFeatures(body, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Get variable by key for user data
     * @param {UserData} body
     * @param {string} key Variable key
     * @param {any} defaultValue Value returned if request fails or doesn't return a value for the key
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getVariableByKey(body: UserData, key: string, defaultValue: any,  options?: any) {
        return DVCFp(this.configuration).getVariableByKey(body, key, defaultValue, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Get all variables by key for user data
     * @param {UserData} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public getVariables(body: UserData, options?: any) {
        return DVCFp(this.configuration).getVariables(body, options)(this.fetch, this.basePath);
    }

    /**
     *
     * @summary Post events to DevCycle for user
     * @param {UserDataAndEventsBody} body
     * @param {*} [options] Override http request option.
     * @throws {RequiredError}
     * @memberof DVC
     */
    public postEvents(body: UserDataAndEventsBody, options?: any) {
        return DVCFp(this.configuration).postEvents(body, options)(this.fetch, this.basePath);
    }

}
